function sequence = load_sequence(seqName, camType, posStart, numOfFrames)
% sequence = load_sequence(seqName, camType, posStart, numOfFrames):
%
%
% input variables:
%
% - seqName: holds the complete path of the directory storing the sequence.
%   The files containing frames of the Swissranger
%   camera has to be named in the following way:
%   frame_0001.dat, frame_0002.dat, ...
%
% - posStart: determines that frame were the sequence should start
%    e.g. 1 for the first frame ...
%
% - numOfFrames: determines the number of frames the sequence should contain
% 
% - camType: specifies the camera used
%    camera types known: 'PMD_16x64', 'PMD_120x160', 'SR_176x144' 

if(seqName(end) ~= filesep)
    seqName = [seqName filesep];
end

if(nargin < 4)
   d = dir(seqName);
   idx = arrayfun(@(x)(~isempty(strfind(x.name, 'frame'))), d, ...
        'UniformOutput', true);
   d = d(idx);
   numOfFrames = length(find([d.isdir] == false));
end

if(nargin < 3)
    posStart = 1;
end

disp('Loading sequence data ...');

sequence = struct('frames', [], 'camType', '');

sequence.camType = generate_camera_data(camType);

seq = [];
frame = struct('distances', [], 'intensities', [], 'amplitudes', [], 'points3D', [], 'points3D_org', [], 'usage', []);

if(strcmp(camType, 'PMD_120x160') || strcmp(camType, 'PMD_16x64'))
    
    currFrame = posStart-1;
    name = '';
    help = [];


    i=1;
    while(i<=numOfFrames)

        %distances in mm

        name = [seqName 'frame' num2str(currFrame)];
        help = load(name);

        frame.distances = 1000 * help(3:end);

        name = [seqName 'frame' num2str(currFrame) '_amp'];
        help = load(name);

        frame.amplitudes = help(3:end);

        name = [seqName 'frame' num2str(currFrame) '_greyimg'];
        help = load(name);

        frame.intensities = help(3:end);

        frame.usage = 1:1:(sequence.camType.height*sequence.camType.width);

        seq = [seq frame];

        i=i+1;
        currFrame = currFrame+1;
    end
    
elseif(strcmp(camType, 'SR_176x144') | strcmp(camType, 'SR4_176x144') | strcmp(camType, 'SR_176x144calib'))
    
    currFrame = posStart;
    name = '';
    help = '';
    fr = [];
    
    i=1;
    while(i<=numOfFrames)
        
        help = num2str(currFrame);
        switch length(help)
            case 1
               help = ['000' help];
            case 2
               help = ['00' help];
            case 3
               help = ['0' help];
            otherwise ;
        end
        
        name = [seqName 'frame_' help '.dat'];
        
        try
            fr = load(name);
        catch
            i = i+1;
            currFrame = currFrame + 1;
            continue;
        end
        
        frame = srdat2frame(fr, sequence.camType);
        
        seq = [seq frame];
        
        currFrame = currFrame + 1;
        i=i+1;
        
    end
    
else

    disp('Unkown camera type!');
    
end
    
sequence.frames = seq;