function clusters = cluster_through_connectedPoints_rgrow(pnts, width, height, camProportion, usage, display)
%% clusters = cluster_through_connectedPoints(pnts, width, height, camProportion, usage, display)
%
% camProportion = pixelsize / focallength
% display: 1 - visualize clusters; 0 - no visulaization (optinal; default: 0)

%% value initialization:

if(nargin < 6)
    display = false;
end

if(nargin < 5)
    usage = 1:1:size(pnts, 2);
end

indx = usage;
workingIndx = [];
clusterIndx = [];
clusters = [];
%threshold = 900;

%% region growing:

i = 1;
while(~isempty(indx))
    
    workingIndx = indx(1);
    indx(1) = [];
    % new cluster
    clusterIndx{i} = []; 
    clusters{i} = [];
    
    % find Points in restPnt-set that in the near neighborhood of the current
    % point
    while(~isempty(workingIndx))
        
        currIndx = workingIndx(1);     
        workingIndx(1) = [];
        clusterIndx{i} = [clusterIndx{i} currIndx];
        
        % find indices of neighbors
        ngbIndx = getNgbIndx(currIndx, width, height, 3); %Swissranger: 8-neighboorhood
        
        ngbIndx = ngbIndx(ismember(ngbIndx, usage));
        
        if(isempty(ngbIndx)) continue; end
        
        diff = pnts(1:3, ngbIndx) - repmat(pnts(1:3, currIndx), 1, length(ngbIndx));
        diff = diff.^2;
        diff = sqrt(sum(diff));
        % two neighboorhood pixels have a different absolute Eculidean distance.
        % It depends on the distance of the real 3D points to the image plane of
        % the camera (z-values):
        % 2 * pixelsize / focallength * z-value
        %threshold = (2 * camProportion * currPnt(3,1))^2; %Swissranger
        threshold = 100; %DC_Tracking
        posNN = find(diff < threshold);
        ngbIndx = ngbIndx(posNN);
        
        workingIndx = [workingIndx ngbIndx];
        workingIndx = unique(workingIndx);
        workingIndx(ismember(workingIndx, clusterIndx{i})) = [];
        
    end
    
    clusters{i} = pnts(:, clusterIndx{i});
    indx(ismember(indx, clusterIndx{i})) = [];
    
    i = i+1;
    
end

%% display results:

if(display)
    
    show_clusters(clusters, 20);
    
end
