function [regions, regionsIndx, edges, edgesIndx] = cluster_through_regions(fr, cam, display)
%% [regions, regionsIndx] = cluster_through_regions(fr, cam, display)
%
% points lying inside connected edges = regions are clustered
% usage: specify which points in 'pnts' will be used
% display: 1 - visualize clusters; 0 - no visulaization (optinal; default: 0)

%% some default settings:
if(nargin < 3)
    display = false;
end

pnts = fr.points3D(1:3,:);
width = cam.width;
height = cam.height;
usage = fr.usage;

%% compute edge image and indices of these edges
se = strel('square',5);
imDis = reshape(pnts(3,:), width, [])';
%imDis = imerode(imdilate(imDis, se), se);
bwDis = edge(imDis, 'canny');
bwDis = imerode(imdilate(bwDis, se), se);

imRegions = edge_region_growing(bwDis);
imRegions = reshape(imRegions', 1, []);
imRegions = imRegions(usage);
pos = 1:1:length(pnts);
pos = pos(usage);

%% find edges in image (are denoted by -2)
posEdges = find(imRegions == -2);
edges = pnts(:, pos(posEdges));
edgesIndx = posEdges;

imRegions(posEdges) = [];
pos(posEdges) = [];

%% region growing
regions = [];
regionsIndx = [];
curr = 0;
while(~isempty(imRegions))
    
    curr = imRegions(1);
    posCurr = find(imRegions == curr);
    
    regions{length(regions)+1} = pnts(:, pos(posCurr));
    regionsIndx{length(regionsIndx)+1} = pos(posCurr);
    
    imRegions(posCurr) = [];
    pos(posCurr) = [];    
    
end

%regions{length(regions)+1} = edges;
%regionsIndx{length(regionsIndx)+1} = posEdges;

if(display)

    show_clusters(regions, 100);

end
