function [n, d] = least_square_plane_fitting(pnts)
%% [n, d] = least_square_plane_fitting(pnts)
% computes the normal and distance (Hesse form) of a plane determined by
% the points "pnts"
% n = A^-1 * b

if(isempty(pnts))
    n = [];
    d = [];
    return;
end

%% compute the components of matrix A:

a11 = sum(pnts(1,:).^2);
a22 = sum(pnts(2,:).^2);
a33 = sum(pnts(3,:).^2);

a12 = sum(pnts(1,:).*pnts(2,:));
a13 = sum(pnts(1,:).*pnts(3,:));
a21 = a12;
a23 = sum(pnts(2,:).*pnts(3,:));
a31 = a13;
a32 = a23;

A = [a11 a12 a13; a21 a22 a23; a31 a32 a33];

%% compute the components of vector b:

b1 = -1 * sum(pnts(1,:));
b2 = -1 * sum(pnts(2,:));
b3 = -1 * sum(pnts(3,:));

b = [b1; b2; b3];

%% solve equation

u = A^-1 * b;

d = 1 / (sum(u.^2));
n = -d * u;