function [posEnd, nEnd, dEnd] = main_plane_extraction(pnts)
%% [posEnd, nEnd, dEnd] = main_plane_extraction(pnts)
%
% pnts: set of points
%           -                 -
%          |p11 p21 p31 p41    |
%   pnts = |p12 p22 p32 p42 ...|
%          |p13 p32 p33 p43    |
%           -                 -

pnts = pnts(1:3,:);

n = size(pnts,2);
tPlane = 60; %Swissrangervalue
% tPlane = 1; %Daimlerdatavalue
tIn = 0.9 * length(pnts);
inlierPnts = [];
maxIter = 40;
minIn = 3;

posEnd = [];
disEnd = [];
nEnd = [];
dEnd = 0;

if(n < 3) return; end

if(n == 3)
    if(iscolinear(pnts))
        return;
    end
end

%% RANSAC-iterations:

counter = 0;
while(1)
    
    if(counter > maxIter) break; end
    
    % select randomly three points
    pos = random_positions(3, n);

    p1 = pnts(1:3, pos(1));
    p2 = pnts(1:3, pos(2));
    p3 = pnts(1:3, pos(3));
    
    if(iscolinear(p1, p2, p3)) 
        counter = counter+1;
        continue; 
    end

    [nk, dk] = plane_computation([p1 p2 p3]);
    
    distances = abs(nk'*pnts - dk);
    pos = find(distances < tPlane);
    
    inlierPnts = pnts(1:3, pos);
    
    % eigenvalues vs. principal component analysis:
    % faster vs. slower
%     cov = inlierPnts * inlierPnts';
%     [vec, val] = eig(cov);
%     [mi, in] = min([val(1,1) val(2,2) val(3,3)]);
%     nk = vec(:, in);
    %[coeff, score, roots] = princomp(inlierPnts');
    %nk = coeff(:,3);
    %dk = nk'* mean(inlierPnts,2);
    
    [nk, dk] = plane_computation(inlierPnts);
    score = zeros(length(inlierPnts), 3);
    score(:,3) = (nk'*inlierPnts - dk)';
    
    % set of inliers too small
    if(length(pos) < minIn)
        counter = counter + 1;
        continue;
    else

        if(length(pos) > tIn)
            posEnd = pos;
            disEnd = abs(score(:,3)');
            nEnd = nk;
            dEnd = dk;
            break;
        end
        
        if(length(pos) > length(posEnd))
            posEnd = pos;
            disEnd = abs(score(:,3)');
            nEnd = nk;
            dEnd = dk;
        end
        
        if(length(pos) == length(posEnd))
            if(median(abs(score(:,3))) < median(disEnd))
                disEnd = abs(score(:,3)');
                posEnd = pos;
                nEnd = nk;
                dEnd = dk;
            end
        end
        
    end
    
    counter = counter+1;
    
end

rest = 1:1:n;
rest(posEnd) = [];

% fig = plot_3Dpoints(pnts(1:3, posEnd), 'r');
% fig = plot_3Dpoints(pnts(1:3, rest), 'k', fig);