function [planesNew, rest, restIndx] = merge_planes(planes, method)
%% planesNew = merge_planes(planes, method)
%
% approach based on region growing
%
% possible methods:
%  - 'mean': uses the mean of the patches
%  - 'median': uses the median of the patches
%  - 'ngbPatches': uses the nearest neighbor patches
%  - 'fastNgbPatches': finds quick nearest neighbor patches 

emp = cellfun('isempty', {planes.pnts});
allPlanes = planes(~emp);
planesNew = [];
rest = [];
restIndx = [];

absoluteNum = length(allPlanes); 

while(~isempty(allPlanes))

    %progressbar(1 - length(allPlanes) / absoluteNum);
    
    currSet = []; 
    workingPlanes = allPlanes(1);
    allPlanes(1) = [];
    
    while(~isempty(workingPlanes))
        
        currPlane = workingPlanes(1);
        currSet = [currSet currPlane];
        workingPlanes(1) = [];
        
        if(isempty(allPlanes) || isempty(currPlane.pnts))
            continue;
        end
        
        if(length(allPlanes) == 1)
            if(isempty(allPlanes.pnts))
                continue;
            end
        end
        
        % conormality measure:
        angles = acos(currPlane.n'*[allPlanes.n]);
        
        % distance measure:
	    if(strcmp(method, 'mean') || strcmp(method, 'median'))
            % cSmoothPlane = smooth_planes(currPlane);
            if(strcmp(method, 'mean'))
                currMean = mean(currPlane.pnts, 2);
            elseif(strcmp(method, 'median'))	        	
                currMean = median(currPlane.pnts, 2);
            else
                disp('Error: method must be "mean" or "median"');
            end

            %currMean = mean(cSmoothPlane.pnts, 2);
            cNormPnts = currPlane.pnts - repmat(currMean, 1, size(currPlane.pnts, 2));
            %cNormPnts = cSmoothPlane.pnts - repmat(currMean, 1, size(cSmoothPlane.pnts, 2));
            currRadius = max(sqrt(sum(cNormPnts.^2)));

            %aSmoothPlanes = smooth_planes(allPlanes);
            allMean = cellfun(@(x) (median(x, 2)), {allPlanes.pnts}, 'UniformOutput', false);
            %allMean = cellfun(@(x) (mean(x, 2)), {aSmoothPlanes.pnts}, 'UniformOutput', false);
            allNormPnts = cellfun(@(x, y) (x - repmat(y, 1, size(x,2))), {allPlanes.pnts}, allMean, 'UniformOutput', false);
            %allNormPnts = cellfun(@(x, y) (x - repmat(y, 1, size(x,2))), {aSmoothPlanes.pnts}, allMean, 'UniformOutput', false);
            allRadius = cellfun(@(x) (max(sqrt(sum(x.^2)))), allNormPnts, 'UniformOutput', false);

            radiusDecider = [allRadius{1:end}] + currRadius;

            allDistMean = [allMean{1:end}] - repmat(currMean, 1, length(allMean));
            distances = sqrt(sum(allDistMean.^2));

            disDecider = distances - radiusDecider;

            % find patches, which fulfill constraints:
            in = find(angles < (1/18 * pi) & disDecider <= 0);

        elseif(strcmp(method, 'ngbPatches'))

            nCurr = size(currPlane.pnts, 2);
            allMins = ones(1, length(allPlanes)) * realmax;
            cAllMins = ones(1, length(allPlanes)) * realmax;

            i = 1;
            while(i<=nCurr)

                currAllMins = cellfun(@(x) (min(sum((x - repmat(currPlane.pnts(1:3,i), 1, size(x,2))).^2))), {allPlanes.pnts}, 'UniformOutput', false);
                cAllMins = [currAllMins{1:end}];

                allMins = min(allMins, cAllMins);

                i = i+1;

            end

            allMins = sqrt(allMins);

            % find patches, which fulfill constraints:
            in = find(angles < (1/18 * pi) & allMins <= 30);

        elseif(strcmp(method, 'fastNgbPatches'))

            candidates = find(angles < (3/36 * pi));

            currMean = mean(currPlane.pnts, 2);
            radius = max(sqrt(sum((currPlane.pnts - repmat(currMean, 1, size(currPlane.pnts, 2))).^2)));
            decider = radius + 120;

            surCand = cellfun(@(x) (any(x(1,:) > currMean(1) - decider & x(1,:) < currMean(1) + decider &  x(2,:) > currMean(2) - decider & x(2,:) < currMean(2) + decider & x(3,:) > currMean(3) - decider & x(3,:) < currMean(3) + decider)), {allPlanes(candidates).pnts}, 'UniformOutput', false);
            surCand = [surCand{1:end}];

            candidates = candidates(surCand);

            candMins = [];
            i = 1;
            while(i<=length(candidates))

                currCells = mat2cell(allPlanes(candidates(i)).pnts, 3, ones(1, size(allPlanes(candidates(i)).pnts, 2)));
                currMin = cellfun(@(x) (min(sqrt(sum((currPlane.pnts - repmat(x, 1, size(currPlane.pnts,2))).^2)))), currCells, 'UniformOutput', false);

                candMins = [candMins min([currMin{1:end}])];

                i = i+1;

            end

            disCand = find(candMins < 120);
            in = candidates(disCand);

        else
            disp('Error: method unknown - must be "mean", "median", or "ngbPatches"!');
        end
        
       
        workingPlanes = [workingPlanes allPlanes(in)];
        allPlanes(in) = [];
       
        
    end
    
    points = [currSet.pnts];
    %indx = [currSet.indx];
    [posEnd, nEnd, dEnd] = main_plane_extraction(points);
    %posEnd = 1:size(points,2);
    %[c,s] = princomp(points(1:3,:)');
    %nEnd = c(:,3);
    %dEnd = nEnd'*mean(points,2);
    
    if(~isempty(nEnd))    
        plane.pnts = points(:, posEnd);
        %plane.indx = indx(posEnd);
        plane.n = nEnd;
        plane.d = dEnd;

        % put outliers of planes to rest
        points(:, posEnd) = [];
        
        if(isfield(currSet, 'indx'))
        indx = [currSet.indx];
        plane.indx = indx(posEnd);
        indx(posEnd) = [];
        restIndx = [restIndx indx];
        end
    
        planesNew = [planesNew plane];
        
    end
    
    %indx(posEnd) = [];
    rest = [rest points];
    %restIndx = [restIndx indx];
    
    %plane.pnts = currSet;
    %coeff = princomp(currSet');
    %plane.n = coeff(:,3);
    %plane.d = plane.n' * mean(currSet, 2);
    
end

%progressbar(1);

% planesNew = [];
% plane = struct('pnts', [], 'n', [], 'd', 0);
% 
% currSet = [];
% 
% while(~isempty(planes))
%     
%     currSet = [];
%     workingPlanes = planes(1);
%     planes(1) = [];
%     
%     while(~isempty(workingPlanes))
%         
%         currPlane = workingPlanes(1);
%         currSet = [currSet currPlane.pnts];
%         workingPlanes(1) = [];
%         
%         if(isempty(planes))
%             continue;
%         end
%         
%         % conormality measure:
%         angles = acos(currPlane.n'*[planes.n]);
%         
%         % coplanarity measure:
%         % currMean = mean(currPlane.pnts, 2);
%         % planesMean = arrayfun(@(x)mean(x.pnts,2), planes, 'UniformOutput', false);
%         % planesMean= cell2mat(planesMean);
% %         currN = size(currPlane.pnts, 2);
% %         l = 1;
% %         while(l <= currN)
% %             
% %             planesCurrR12 = arrayfun(@(x)(x.pnts - repmat(currPlane.pnts(1:3,l), 1, size(x.pnts,2))), planes, 'UniformOutput', false);
% %             
% %             l = l+1;
% %             
% %         end
% 
%         r12 = zeros(3, length(planes));
% 
%         l = 1;        
%         while(l <= length(planes))
%             
%             k = 1;
%             currR12 = [];
%             currLength = realmax;
%             while(k <= size(currPlane.pnts, 2))
%                 
%                 helpR12 = planes(l).pnts - repmat(currPlane.pnts(1:3,k), 1, size(planes(l).pnts, 2));
%                 helpLengths = sum(helpR12.^2);
%                 [mi, indx] = min(helpLengths);
%                 
%                 if(mi < currLength)
%                     currR12 = helpR12(:, indx);
%                     currLength = mi;
%                 end
%                 
%                 k = k+1;
%                 
%             end
%             
%             r12(:,l) = currR12;
%             
%             l = l+1;
%             
%         end
% 
%         % r12 = planesMean - repmat(currMean, 1, size(planesMean, 2));
%         
%         firstTerm = abs(currPlane.n'*r12);
%         secondTerm = zeros(1, size(r12, 2));
%         for j = 1:size(r12, 2)
%             secondTerm(j) = abs(planes(j).n'*r12(:,j));
%         end
%         
%         decider = max([firstTerm; secondTerm]);
%         %threshold = 8 * 0.005 * currMean(3);
%         threshold = 30;
%         in = find(angles < (2/18 * pi) & decider < threshold);
%         
% %         ds = abs([planes.d] - currPlane.d);
% %         in = find(angles < (2/18 * pi) & ds < 40);
%         
%         workingPlanes = [workingPlanes planes(in)];
%         planes(in) = [];
%         
%     end
%     
%     plane.pnts = currSet;
%     coeff = princomp(currSet');
%     plane.n = coeff(:,3);
%     plane.d = plane.n' * mean(currSet, 2);
%     
%     planesNew = [planesNew plane];
%     
% end
        
