function [n, d] = plane_computation(pnts)
% [n, d] = plane_computation(pnts)
% 
% n: normal vector of E
%        -  -
%        |n1|
%    n = |n2|
%        |n3|
%        -  -
% d: distanz of E to the origin
% p1, p2, p3: points in E (if #pnts = 3)
%         -   -       -   -       -   -
%         |p11|       |p21|       |p31|
%    p1 = |p12|  p2 = |p22|  p3 = |p32|
%         |p13|       |p23|       |p33|
%         -   -       -   -       -   -
% 
% the plane is described in Hesse normal form:
%    ->  ->           ->
% E: n * x = d, where n is the normal vector of E

pnts = pnts(1:3,:);

n = [];
d = 0;

if(size(pnts,2) < 3)
    
    disp('Error: not enough pnts for plane computation');
    return;
    
elseif(size(pnts,2) == 3)
    
    p1 = pnts(:,1);
    p2 = pnts(:,2);
    p3 = pnts(:,3);

    a = p2 - p1;
    b = p3 - p1;

    n = cross(a, b);
    n = n / norm(n);

    d = n'*p1;
    
else
    
    coeff = princomp(pnts');
    n = coeff(:,3);
    d = n'*mean(pnts,2);
    
end
