function planes = pnts2plane_classification_dac(pnts, display)
%% planes = pnts2plane_classification_dac(pnts)
%
% reference:
% Jan Weingarten et al: " A Fast and Robust 3D Feature Extraction Algorithm
%                         for Structured Environment Reconstuction"

%% some defaults:
if(nargin < 2)
    display = false;
end

%% discretization of the point cloud
voxels = generate_voxels(pnts, [60 60 60]);

%% segmentation
pos = cellfun(@(x)(main_plane_extraction(x(1:3,:))), voxels, 'UniformOutput', false);
voxels_new = cellfun(@(x,i)(x(:,i)), voxels, pos, 'UniformOutput', false);

voxels_new = voxels_new(~cellfun('isempty', voxels_new));

%[n, d] = cellfun(@(x)(least_square_plane_fitting(x)), voxels_new, 'UniformOutput', false);
coeff = cellfun(@(x)(princomp(x(1:3,:)')), voxels_new, 'UniformOutput', false);
n = cellfun(@(x)(x(:,3)), coeff, 'UniformOutput', false);
center = cellfun(@(x)(mean(x(1:3,:),2)), voxels_new, 'UniformOutput', false);
d = cellfun(@(x,y)(x'*y), n, center, 'UniformOutput', false);


help = [n'; d'; center'; voxels_new';
    repmat({[1 0 0]},1,length(d)); repmat({30},1,length(d))];

patches = cell2struct(help, ...
        {'normal', 'd', 'center', 'pnts', 'color', 'radius'}, 1)';

%help = [n'; d'; voxels_new'];
%planes = cell2struct(help, {'n', 'd', 'pnts'}, 1)';
%
%planes = smooth_planes(planes);

%% region growing:


a_thresh = 1/18 * pi;
%d_thresh = 70; 

allIndx = [1:length(patches)];
clusters = [];

while(~isempty(allIndx))
    
    currSet = [];
    workingIndx = allIndx(1);
    allIndx(1) = [];
    
    while(~isempty(workingIndx))
        
        currIndx = workingIndx(1);
        currSet = [currSet patches(currIndx).pnts];
        workingIndx(1) = [];
        
        d_thresh = 4 * 0.05 * mean(patches(currIndx).pnts(3,:), 2);
        
        if(isempty(allIndx))
            continue;
        end
        
        angles = acos(patches(currIndx).normal'*[patches(allIndx).normal]);
        in1 = find(angles < a_thresh);
        
        dis = abs(patches(currIndx).normal'*[patches(allIndx).center] - ...
            repmat(patches(currIndx).normal'*patches(currIndx).center, 1, length(allIndx)));
        in2 = find(dis < d_thresh);
        
%          % coplanarity measure:
%         currPnt = patches(currIndx).center;
%         currNormal = patches(currIndx).normal;
%         ngbPnts = [patches(allIndx).center];
%         ngbNormals = [patches(allIndx).normal];
%         
%         r12 = ngbPnts - currPnt * ones(1, size(ngbPnts, 2));
%         
%         firstTerm = abs(currNormal'*r12);
%         secondTerm = zeros(1, size(r12, 2));
%         for j = 1:size(r12, 2)
%             secondTerm(j) = abs(ngbNormals(:,j)'*r12(:,j));
%         end
%         
%         decider = max([firstTerm; secondTerm]);
%         
%         in2 = find(decider < d_thresh);
        
        in = intersect(in1, in2);
        
        workingIndx = [workingIndx allIndx(in)];
        allIndx(in) = [];
        
    end
    
    clusters = [clusters {currSet}];
    
end

planes = clusters;
        
%% display results:

if(display)
    show_clusters(clusters, 25);
end
        