function [planes, rest] = pnts2plane_classification_vgrow(pnts, display)
%% [planes, rest] = pnts2plane_classification_rgrow(pnts, display)
%
%   The function is based on the paper "Geometry and Texture Recovery" 
%   of Stamos and Allen clustering arbitrary point clouds. 
%   Use of region growing techniques.
%
% pnts: set of points with the following structure
%           | p1x p2x p3x p4x     |
%    pnts = | p1y p2y p3y p4y ... |
%           | p1z p2z p3z p4z     |
% display: 1 - visualize clusters; 0 - no visualization (optional; default: 0)

%% some default values:

if(nargin < 2)
    display = false;
end

rest = [];

%% generate oriented particels:
% [VoxelPoints, voxelWithPoint, VoxelColors, n, nbrVoxel, startingPoint] = ...
%     generate_voxels(pnts, [20 20 20], 'r');
% 
% OrientedParticles = calc_plane(voxelWithPoint, VoxelPoints, VoxelColors, n, ...
%     nbrVoxel, [20 20 20], startingPoint, 'Ransac');

OP = generate_oriented_particles(pnts);

%% merging:
%[patchesNew, patches, rest] = cluster_merging(OrientedParticles,'Stamos');

% classify if points are planar or not:

deviation = [OP.dev];
%thres_dev = 0.67 * mean(deviation);
thres_dev = 15;
planarIndx = find(deviation <= thres_dev); %Swissranger
notPlanarIndx = find(deviation > thres_dev); %Swissranger

rest = [rest [OP(notPlanarIndx).center]];

allIndx = planarIndx;


%allPatches = OrientedParticles;
%patchesNew = [];

clusters = {};

absoluteNum = size(allIndx,2);

% region growing:

while(~isempty(allIndx))

    progressbar(1 - size(allIndx,2) / absoluteNum);

    currSet = [];
    currNormal = [];
    currColor =  [];

    workingIndx = allIndx(1);
    allIndx(1) = [];

    while(~isempty(workingIndx))

        currIndx = workingIndx(1);
        currSet = [currSet OP(currIndx).center];
        %currNormal = [currNormal currPatch.normal];http://www.spiegel.de/politik/ausland/0,1518,543512,00.html
        %currColor = [currColor currPatch.color];
        %workingPatches(randnumberworking) = [];
        workingIndx(1) = [];
       

        if(isempty(allIndx))
            continue;
        end
 
        % for redundant point elimination:
        % a_thresh = 1/18* pi;
        % d_thresh = 2;

        % for plane estalishing on arbitrary point clouds
        a_thresh = 3/18 * pi;
        d_thresh = 120; %Swissranger
        
        ngb = intersect(allIndx, OP(currIndx).ngb); 
        
        if(isempty(ngb))
            continue;
        end

        % conormality measure:
        angles = acos(OP(currIndx).normal'*[OP(ngb).normal]);
        % find patches, which fulfill constraints:
        in1 = find(angles < a_thresh);
        if (isempty(in1))
           % patchesNew = [patchesNew currPatch];

           continue; 
        end
        
        ngb = ngb(in1);
        
        % coplanarity measure:        
        r12 = [OP(ngb).center] - OP(currIndx).center * ones(1, size(ngb, 2));
        
        firstTerm = abs(OP(currIndx).normal'*r12);
        secondTerm = zeros(1, size(r12, 2));
        for j = 1:size(r12, 2)
            secondTerm(j) = abs(OP(ngb(j)).normal'*r12(:,j));
        end
        
        decider = max([firstTerm; secondTerm]);
        in2 = find(decider < d_thresh);

        if (isempty(in2))
           % patchesNew = [patchesNew currPatch];
           continue; 
        end
        in = ngb(in2);

        workingIndx = [workingIndx in];
        workingIndx = unique(workingIndx);
        allIndx(ismember(allIndx, in)) = [];

    end
    
    clusters = [clusters, {currSet}];
    %patch.center = mean(currSet,2);
    %patch.color = mean(currColor,2);
    %coeff = princomp(currSet');
    %patch.normal = coeff(:,3);
    
    %patch.normal = mean(currNormal,2);
    
    % radius = max(sqrt(sum((currSet-repmat(currPatch.center,1,size(currSet,2))).^2)));
    %d = patch.normal' * center;
    %patch.center = center - ((patch.normal'*center)-d)*patch.normal;
    %radius = max(sqrt(sum((currSet-repmat(patch.center,1,size(currSet,2))).^2)));
    %if radius < currPatch.radius
    %    patch.radius = currPatch.radius;
    %else
    %    patch.radius = radius;
    %end
    
    %patchesNew = [patchesNew patch];

end

progressbar(1);
    
%% display results:

if(display)
    fig = show_clusters(clusters, 25);
    fig = plot_3Dpoints(rest, 'k', fig);
end

planes = clusters;