function ngbIndx = getNgbIndx(indx, width, height, nHood, woC, pos)
%% ngbIndx = getNgbIndx(indx, width, height, nHood, woIndx)
% 
% Computes the sequential indices (row-wise, not column-wise as default in
% Matlab) of an arbitrary neighborhood
%
% If no nHood is specified the default value is 3
% If woC = true, then current center (specified by indx) is removed, else center
% is kept. The default is set to true.

%% some default values:

if(nargin < 6);
    accelerate = false;
else
    accelerate = true;
end

if(nargin < 5)
    woC = true;
end

if(nargin < 4)
    nHood = 3;
end

%% computing the neighboring indices:

[currR, currC] = sindx2mindx(indx, width, height);

half = (nHood-1)/2;
h1 = floor(half);
h2 = ceil(half);

if(accelerate)
    
    rs = currR-h1;
    if(rs < 1) rs = 1; end
    
    re = currR+h2;
    if(re > height) re = height; end
    
    cs = currC-h1;
    if(cs < 1) cs = 1; end
    
    ce = currC+h2;
    if(ce > width) ce = width; end
    
    ngbIndx = pos(rs:re, cs:ce);
    ngbIndx = reshape(ngbIndx', 1, []);
    
else
    
    r = currR-h1:currR+h2;
    r(find(r<1 | r>height)) = [];
    c = currC-h1:currC+h2;
    c(find(c<1 | c>width)) = [];
    
    ngb = [sort(repmat(r, 1, length(c))); repmat(c, 1, length(r))];
    ngbIndx = mindx2sindx(ngb(1,:), ngb(2,:), width, height);
    %ngbIndx = unique(ngbIndx);
    
end

if(woC)
    ngbIndx(find(ngbIndx == indx)) = [];
end