function [area, shape, b1, b2, s1min, s1max, s2min, s2max] = get_expansion(pnts, display)
%% [area, shape, b1, b2, s1min, s1max, s2min, s2max] = get_expansion(pnts, display)

if(nargin < 2)
    display = false;
end

%% rotate plane so that it is parallel to xy-plane and main principle axis
%  are parallel to the x-/y-axis

coeff = princomp(pnts(1:3,:)');
b1 = coeff(:,1);
b2 = coeff(:,2);
n = coeff(:,3);

if(all(n == [0;0;1]))
    pnts1 = pnts - repmat(mean(pnts,2), 1, size(pnts,2));
else
    pnts1 = align_pnts2plane(pnts, n, mean(pnts,2));
end

try
    k = convhull(pnts1(1,:), pnts1(2,:));
catch
    k = 1:size(pnts1,2);
end

c = princomp(pnts1(1:2,k)');
b = c(:,1);
r = [b(2) -b(1); b(1) b(2)];
R2 = eye(4);
R2(1:2,1:2) = r;
pnts2 = R2*pnts1;
c2 = princomp(pnts2(1:2,k)');

%% compute the expansion along the principle axis -> area and shape feature

[m, i1] = max(abs(c2(:,1)));
[m, i2] = max(abs(c2(:,2)));

psort_i1 = sort(pnts2(i1,:));
psort_i2 = sort(pnts2(i2,:));

s1min = min(abs(psort_i1(1)), abs(psort_i1(end)));
s1max = max(abs(psort_i1(1)), abs(psort_i1(end)));
s2min = min(abs(psort_i2(1)), abs(psort_i2(end)));
s2max = max(abs(psort_i2(1)), abs(psort_i2(end)));

area = (s1min+s1max)*(s2min+s2max);
shape = min((s2min+s2max),(s1min+s1max))/max((s2min+s2max),(s1min+s1max));

%% show expansion

if(display)
    
    fig = figure('Position', [1000 1280 900 900]);
    %subplot(1,2,1);
    %plot_3Dpoints(pnts, 'k', fig);
    %subplot(1,2,2);
    plot(pnts2(1,:), pnts2(2,:), '.'); axis equal; hold on;
    quiver(0,0,c2(1,1),c2(2,1), -s1max, 'Color', 'r');
    quiver(0,0,c2(1,2),c2(2,2), -s2max, 'Color', 'g');
    %rectangle('Position', [-s2max -s1max 2*s2max 2*s1max], 'Curvature', [1 1]);
    %rectangle('Position', [-s2min -s1min 2*s2min 2*s1min], 'Curvature', [1 1]);
    rectangle('Position', [psort_i2(1) psort_i1(1) (s2min+s2max) (s1min+s1max)]);
    
end

